class Payload(object):

    def __eq__(self, other):
        return self.payload_str == other.payload_str and other.check_type_list == other.check_type_list

    def __hash__(self):
        return hash((self.payload_str, str(self.check_type_list)))

    def __str__(self):
        return self.payload_str

    def __init__(self, payload_str, check_type_list = []):

        self.check_type_list = check_type_list
        self.payload_str = payload_str

if __name__ == "__main__":

    p1 = Payload("ddd", ["dddd"])
    p2 = Payload("ddd", ["dddd"])

    if p1 == p2:
        print "equal"



#!/usr/bin/python2.7

import json
import sys
import urlparse
import traceback

for line in sys.stdin:

    line = line.strip()
    domain_in, result_dic_in = line.split("\t")
    print domain_in + "\t" + result_dic_in



import time
import json
import requests
import sys
from urlparse import parse_qs
from urlparse import urlparse
from urlparse import urlunparse
from urllib import urlencode
from multiprocessing import Process, Pool, Queue, current_process
from multiprocessing.pool import ThreadPool
from multiprocessing import TimeoutError
import traceback
from sets import Set

def replace_param_value(url, param, replacement_string):
    '''Replace a parameter in a url with another string. Returns
    a fully reassembled url as a string.'''

    url_parsed = urlparse(url)
    query_dic = parse_qs(url_parsed.query)
    query_dic[param] = replacement_string

    #this incidentally will also automatically url-encode the payload (thanks urlencode!)
    query_reassembled = urlencode(query_dic, doseq = True)

    #3rd element is always the query, replace query with our own
    url_list_parsed = list(url_parsed)
    url_list_parsed[4] = query_reassembled
    url_parsed_q_replaced = tuple(url_list_parsed)
    url_reassembled = urlunparse(url_parsed_q_replaced)

    return url_reassembled

def generate_fuzzy_urls(url, payloads):

    parsed_url = urlparse(url)
    parsed_url_query = parsed_url.query
    url_q_dic = parse_qs(parsed_url_query)

    fuzzy_urls = []
    for query_param, query_val in url_q_dic.iteritems():

        for payload in payloads:
            fuzzy_urls.append(replace_param_value(url, query_param, str(payload)))

    return fuzzy_urls



from massweb.payloads.payload import Payload
from massweb.targets.target import Target

class FuzzyTarget(Target):

    def __eq__(self, other):
        return self.url == other.url and self.payload == other.payload and self.ttype == other.ttype

    def __hash__(self):
        return hash((self.url, self.payload, self.ttype))

    def __init__(self, url, fuzzy_param, ttype = "get", data = None, payload = None):

        if type(payload) != Payload:
            raise Exception("payload must be of type Payload")

        super(FuzzyTarget, self).__init__(url, data = data, ttype = ttype)
        self.payload = payload
        self.fuzzy_param = fuzzy_param

if __name__ == "__main__":

    ft1 = FuzzyTarget("ddd", payload = Payload("ddd", check_type_list = ["ddd"]))
    ft2 = FuzzyTarget("ddd", payload = Payload("ddd", check_type_list = ["ddd"]))

    if ft1 == ft2:
        print "equal"

    print ft1.payload
    print ft1.url
    print ft1



from massweb.payloads.payload import Payload
from massweb.targets.target import Target

class CrawlTarget(Target):

    def __init__(self, url, data = None, ttype = "get", status = "unfetched"):

        super(CrawlTarget, self).__init__(url, data = data, ttype = ttype)
        self.status = status

if __name__ == "__main__":

    ct = CrawlTarget("dddd")

    print ct.ttype



from massweb.vuln_checks.match import match_strings
from massweb.vuln_checks.check import Check

class XPathICheck(object):

    def __init__(self):

        vuln_strings_raw = ['System.Xml.XPath.XPathException:',
        'MS.Internal.Xml.',
        'Unknown error in XPath',
        'org.apache.xpath.XPath',
        'A closing bracket expected in',
        'An operand in Union Expression does not produce a node-set',
        'Cannot convert expression to a number',
        'Document Axis does not allow any context Location Steps',
        'Empty Path Expression',
        'DOMXPath::'
        'Empty Relative Location Path',
        'Empty Union Expression',
        "Expected ')' in",
        'Expected node test or name specification after axis operator',
        'Incompatible XPath key',
        'Incorrect Variable Binding',
        'libxml2 library function failed',
        'libxml2',
        'xmlsec library function',
        'xmlsec',
        "error '80004005'",
        "A document must contain exactly one root element.",
        '<font face="Arial" size=2>Expression must evaluate to a node-set.',
        "Expected token ']'",
        "<p>msxml4.dll</font>",
        "<p>msxml3.dll</font>",
        '4005 Notes error: Query is not understandable']
        self.vuln_strings = [x.lower() for x in vuln_strings_raw]

    def check(self, content):
        
        content = content.lower()
        return match_strings(content, self.vuln_strings)



from massweb.vuln_checks.match import parse_match
from massweb.vuln_checks.check import Check

class XSSCheck(Check):

    def __init__(self):

        self.vuln_string = "alert(31337)"

    def check(self, content):
        
        return parse_match(content, "script", self.vuln_string)



import sys
import requests
import traceback
from bs4 import BeautifulSoup, SoupStrainer

def match_string(_input, match):

    if match in _input:
        return True

    else:
        return False

def match_strings(_input, match_list):

    for match in match_list:
        if match in _input:
            return True

    return False

def parse_match(_input, tag, match):

    if sys.getsizeof(_input) > 4097152:
        sys.stderr.write("Input is too big to parse, skipping it")
        return False

    for script in BeautifulSoup(_input, 'html5lib', 
                                parse_only = SoupStrainer([tag])):

        try:
            script_text = script.get_text()
            if match in script_text:
                return True

        except:
            continue

    return False

if __name__ == "__main__":

    x = requests.get("http://www.punkspider.org/lists/whitelist.list").text
    print parse_match(x, "script", "t")



import sys
from copy import deepcopy
from massweb.targets.target import Target
from massweb.targets.fuzzy_target import FuzzyTarget
from massweb.fuzzers.ifuzzer import iFuzzer
from massweb.fuzz_generators.url_generator import generate_fuzzy_urls
from massweb.mass_requests.mass_request import MassRequest
from massweb.payloads.payload import Payload
from urlparse import parse_qs
from urlparse import urlparse
from urlparse import urlunparse
from urllib import urlencode
from massweb.vuln_checks.mxi import MXICheck
from massweb.vuln_checks.osci import OSCICheck
from massweb.vuln_checks.sqli import SQLICheck
from massweb.vuln_checks.trav import TravCheck
from massweb.vuln_checks.xpathi import XPathICheck
from massweb.vuln_checks.xss import XSSCheck
from massweb.results.result import Result

class WebFuzzer(iFuzzer):

    def __init__(self, targets = [], payloads = [], num_threads = 10, time_per_url = 10, request_timeout = 10, proxy_list = [{}]):

        self.mreq = MassRequest(num_threads = num_threads, time_per_url = time_per_url, request_timeout = request_timeout, proxy_list = proxy_list)
        self.targets = targets
        self.payloads = payloads
        self.mxi_check = MXICheck()
        self.osci_check = OSCICheck()
        self.sqli_check = SQLICheck()
        self.trav_check = TravCheck()
        self.xpathi_check = XPathICheck()
        self.xss_check = XSSCheck()

    def __replace_param_value(self, url, param, replacement_string):
        '''Replace a parameter in a url with another string. Returns
        a fully reassembled url as a string.'''

        url_parsed = urlparse(url)
        query_dic = parse_qs(url_parsed.query)
        query_dic[param] = replacement_string

        #this incidentally will also automatically url-encode the payload (thanks urlencode!)
        #!might cause some incorrect queries with utf-8, needs more testing
        str_query_dic = {}
        for k, v in query_dic.iteritems():
            str_query_dic[k] = unicode(v).encode('utf-8', 'replace')

        query_reassembled = urlencode(str_query_dic, doseq = True)

        #3rd element is always the query, replace query with our own
        url_list_parsed = list(url_parsed)
        url_list_parsed[4] = query_reassembled
        url_parsed_q_replaced = tuple(url_list_parsed)
        url_reassembled = urlunparse(url_parsed_q_replaced)

        return url_reassembled

    def __generate_fuzzy_target_get(self, target):

        url = target.url
        parsed_url = urlparse(url)
        parsed_url_query = parsed_url.query
        url_q_dic = parse_qs(parsed_url_query)

        fuzzy_targets = []
        for query_param, query_val in url_q_dic.iteritems():

            for payload in self.payloads:

                fuzzy_url = (self.__replace_param_value(url, query_param, str(payload)))
                fuzzy_target = FuzzyTarget(fuzzy_url, query_param, "get", payload = payload)
                fuzzy_targets.append(fuzzy_target)

        return fuzzy_targets

    def __generate_fuzzy_target_post(self, target):

        url = target.url
        fuzzy_targets = []
        post_keys = target.data.keys()

        for key in post_keys:
            data_copy = target.data.copy()

            for payload in self.payloads:
                data_copy[key] = str(payload)
                fuzzy_target = FuzzyTarget(url, key, "post", data = data_copy.copy(), payload = payload)
                fuzzy_targets.append(fuzzy_target)

        return fuzzy_targets
    
    def determine_posts_from_targets(self, dedupe = True):

        self.mreq.get_post_requests_from_targets(self.targets)
        identified_posts = self.mreq.identified_post_requests

        #dedupe posts if relevant
        identified_posts = list(set(identified_posts))

        for ip in identified_posts:
            if ip not in self.targets:
                self.targets.append(ip)

    def generate_fuzzy_targets(self):

        if len(self.targets) == 0:
            raise Exception("Targets list must not be empty!")

        self.fuzzy_targets = []
        for target in self.targets:

            if target.ttype == "get":
                fuzzy_target_list = self.__generate_fuzzy_target_get(target)
                self.fuzzy_targets += fuzzy_target_list

            if target.ttype == "post":
                fuzzy_target_list = self.__generate_fuzzy_target_post(target)
                self.fuzzy_targets += fuzzy_target_list
            
        return self.fuzzy_targets

    def fuzz(self):

        self.mreq.request_targets(self.fuzzy_targets)
        results = []
        for r in self.mreq.results:
            ftarget = r[0]
            #!not yet multithreaded, should it be?
            try:
                result = self.analyze_response(ftarget, r[1].text)
            except:
                #if request failed and str is returned instead of Response obj
                #could save some cycles here not analyzing response
                result = self.analyze_response(ftarget, r[1])

            results.append(result)

        return results

    def analyze_response(self, ftarget, response):

        result_dic = {}
        check_type_list = ftarget.payload.check_type_list
        if "mxi" in check_type_list:
            mxi_result = self.mxi_check.check(response)
            result_dic["mxi"] = mxi_result

        if "sqli" in check_type_list:
            sqli_result = self.sqli_check.check(response)
            result_dic["sqli"] = sqli_result

        if "xpathi" in check_type_list:
            xpathi_result = self.xpathi_check.check(response)
            result_dic["xpathi"] = xpathi_result

        if "trav" in check_type_list:
            trav_result = self.trav_check.check(response)
            result_dic["trav"] = trav_result

        if "osci" in check_type_list:
            osci_result = self.osci_check.check(response)
            result_dic["osci"] = osci_result

        if "xss" in check_type_list:
            xss_result = self.xss_check.check(response)
            result_dic["xss"] = xss_result

        
        return Result(ftarget, result_dic)

if __name__ == "__main__":

    xss_payload = Payload('"><ScRipT>alert(31337)</ScrIpT>', check_type_list = ["xss"])
    trav_payload = Payload('../../../../../../../../../../../../../../../../../../etc/passwd', check_type_list = ["trav"])
    sqli_xpathi_payload = Payload("')--", check_type_list = ["sqli", "xpathi"])

    wf = WebFuzzer()
    wf.add_payload(xss_payload)
    wf.add_payload(trav_payload)
    wf.add_payload(sqli_xpathi_payload)

#    wf.add_target_from_url(u"http://course.hyperiongray.com/vuln1")
#    wf.add_target_from_url(u"http://course.hyperiongray.com/vuln2/898538a7335fd8e6bac310f079ba3fd1/")
    wf.add_target_from_url(u"http://www.wpsurfing.co.za/?feed=%22%3E%3CScRipT%3Ealert%2831337%29%3C%2FScrIpT%3E")
    wf.add_target_from_url(u"http://www.sfgcd.com/ProductsBuy.asp?ProNo=1%3E&amp;ProName=1")
#    wf.add_target_from_url(u"http://www.gayoutdoors.com/page.cfm?snippetset=yes&amp;typeofsite=snippetdetail&amp;ID=1368&amp;Sectionid=1")
    wf.add_target_from_url(u"http://www.dobrevsource.org/index.php?id=1")

    print "Targets list pre post detrmination:"
    for target in wf.targets:
        print target

    print "Targets list after additional injection points have been found:"
    wf.determine_posts_from_targets()
    for target in wf.targets:
        print target.url, target.data

    print "FuzzyTargets list:"
    wf.generate_fuzzy_targets()
    for ft in wf.fuzzy_targets:
        print ft, ft.ttype, ft.data

    print "Results of our fuzzing:"
    for r in wf.fuzz():
        print r, r.fuzzy_target.ttype, r.fuzzy_target.payload

#    print "targs"
#    for target in wf.targets:
#        print target

#    print "--------fuzzy"
#    for target in ft:
#        print target


#    for target in wf.generate_fuzzy_targets():
#        print target.url, str(target.data)

#    for r in wf.fuzz():
#        print r, r.fuzzy_target.ttype, r.fuzzy_target.payload

#    gf = GetFuzzer(proxy_list = [{}])
#    mx_sqli_xmli_trav_osci_payload = Payload("../../../../../../../../../../../../../../../../../../../../etc/passwd#--'@!\\"
#                                             , check_type_list = ["mxi", "sqli", "xpathi", "trav", "osci"])
#

#    gf.add_payload(mx_sqli_xmli_trav_osci_payload)
#    gf.add_payload(xss_payload)

#    gf.add_target_from_url("http://www.hyperiongray.com/?q=user&t=eke")
#    gf.add_target_from_url("http://www.sfgcd.com/ProductsBuy.asp?ProNo=%22%3E%3CSCrIpT%3Ealert%2826702%29%3C%2FScRiPt%3E&amp;ProName=%C2%A2%C3%81%C2%A03%083D%09")
#    gf.add_target_from_url("http://www.gayoutdoors.com/page.cfm?snippetset=yes&amp;typeofsite=snippetdetail&amp;ID=1368&amp;Sectionid=%27%29")
#    gf.add_target_from_url("http://www.dobrevsource.org/index.php?id=..%2F..%2F..%2F..%2F..%2F..%2F..%2F..%2F..%2F..%2F..%2F..%2F..%2F..%2F..%2F..%2F..%2F..%2F..%2F..%2F..%2F..%2Fetc%2Fpasswd")
#    gf.add_target_from_url("http://www.dobrevsource.org/")
#    gf.add_target_from_url("http://www.wpsurfing.co.za/?feed=%22%3E%3CScRipT%3Ealert%2831337%29%3C%2FScrIpT%3E")
#
#    for t in gf.generate_fuzzy_targets():
#        print t.url
#        print t.payload.check_type_list
#
#    for res in gf.fuzz():
#        print res



from massweb.vuln_checks.match import match_strings
from massweb.vuln_checks.check import Check

class OSCICheck(Check):

    def __init__(self):

        self.vuln_string = ["root:x:", "[font]"]

    def check(self, content):
        
        content = content.lower()
        return match_strings(content, self.vuln_string)



#!/usr/bin/python
# -*- coding: utf-8 -*-
import sys
import traceback
from urlparse import urlparse
from bs4 import BeautifulSoup, SoupStrainer
from massweb.mass_requests.mass_request import MassRequest
from massweb.pnk_net.find_post import normalize_link
from massweb.targets.crawl_target import CrawlTarget
from massweb.pnk_net.find_post import find_post_requests
import codecs
import logging
from logging import StreamHandler
logging.basicConfig(format='%(asctime)s %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p')
mc_logger = logging.getLogger('MassCrawlLogger')
mc_logger.setLevel(logging.INFO)

class MassCrawl(object):

    def __init__(self, seeds = [], add_seeds_to_scope = True):

        mc_logger.info("Insantiating MassCrawl object")
        self.seeds = seeds
        self.domains = []
        self.posts_identified = []

        for seed in seeds:
            domain_raw = urlparse(seed).netloc
            if ":" in domain_raw:
                domain = domain_raw.split(":")[0]
            else:
                domain = domain_raw

            self.domains.append(domain)

        self.targets = []
        for seed in seeds:
            ct = CrawlTarget(seed)
            self.add_target(ct)

    def get_domain_from_url(self, url):

        domain_raw = urlparse(url).netloc
        if ":" in domain_raw:
            domain = domain_raw.split(":")[0]
        else:
            domain = domain_raw

        return domain

    def add_to_scope_from_url(self, url):

        domain = self.get_domain_from_url(url)
        self.add_to_scope(domain)

    def add_to_scope(self, domain):

        if domain not in self.domains:
            self.domains.append(domain)

    def in_scope(self, url):

        domain = self.get_domain_from_url(url)

        if domain in self.domains:
            return True
        else:
            return False

    def add_target(self, target):

        if target not in self.targets:
            self.targets.append(target)

    def parse_response(self, response, stay_in_scope = True, max_links = 10):
        #implement max_links

        response_text = response.text
        url = response.url

        link_c = 0
        
        for link in BeautifulSoup(response_text, 'html.parser', parse_only=SoupStrainer(['a', 'img', 'script', 'link'])):

            #stop spitting back links if max links reached
            if link_c > max_links:
                raise StopIteration

            href = None
            if link.get('href'):
                href = link.get('href')

            elif link.get('src'):
                href = link.get('src')

            if href and not href.startswith("mailto:"):

                try:
                    link_normed = normalize_link(href, url)["norm_url"]
                    if stay_in_scope:

                        if self.in_scope(link_normed):

                            link_c += 1
                            yield link_normed

                    else:
                        link_c += 1
                        yield link_normed

                except:
                    traceback.print_exc()
                    continue

    def fetch(self, num_threads = 10, time_per_url = 10, request_timeout = 10, proxy_list = [{}]):
        """Fetch URLs and append them to the seed list"""
        
        self.mreq = MassRequest(num_threads = num_threads, time_per_url = time_per_url, request_timeout = request_timeout, proxy_list = proxy_list)
        unfetched_targets = [unfetched_target for unfetched_target in self.targets if unfetched_target.status == "unfetched"]

        for ut in unfetched_targets:
            mc_logger.info(u"Fetching " + unicode(ut))

        #!note this only fetches via GET, doesn't submit forms for more links
        self.mreq.get_targets(self.targets)
        self.results = self.mreq.results

        for target in self.targets:
            target.status = "fetched"

    def parse(self, stay_in_scope = True):

        for result in self.results:

            try:
                #skip 400s and strings (i.e. failed requests)
                response = result[1]
                response.raise_for_status()

                post_request_targets = find_post_requests(response.url, response.text)
                for target_post in post_request_targets:
                    ct_post = CrawlTarget(target_post.url)
                    ct_post.__dict__ = target_post.__dict__
                    ct_post.status = "unfetched"
                    self.add_target(ct_post)

                links = self.parse_response(response, stay_in_scope = stay_in_scope)

                for link in links:
                    ct_link = CrawlTarget(unicode(link))
                    self.add_target(ct_link)

            except:
                traceback.print_exc()

            mc_logger.info(u"Finished attempted parsing for " + unicode(result[0]))

    def crawl(self, depth = 3, num_threads = 10, time_per_url = 10, request_timeout = 10, proxy_list = [{}]):

        for _ in range(depth):

            mc_logger.info("Entering the fetch phase at depth %s" % str(depth))
            self.fetch()
            mc_logger.info("Entering the parse phase at depth %s" % str(depth))
            self.parse()

if __name__ == "__main__":

    seeds = ["http://www.hyperiongray.com", "http://course.hyperiongray.com/vuln1", "http://course.hyperiongray.com/vuln2/898538a7335fd8e6bac310f079ba3fd1/",
             "http://www.wpsurfing.co.za/?feed=%22%3E%3CScRipT%3Ealert%2831337%29%3C%2FScrIpT%3E", "http://www.sfgcd.com/ProductsBuy.asp?ProNo=1%3E&amp;ProName=1",
             "http://www.gayoutdoors.com/page.cfm?snippetset=yes&amp;typeofsite=snippetdetail&amp;ID=1368&amp;Sectionid=1", "http://www.dobrevsource.org/index.php?id=1",
             u"http://JP納豆.例.jp/", u"http://prisons.ir/", "http://www.qeng.ir/"]

    seeds_uni = [unicode(seed) for seed in seeds]

    mc = MassCrawl(seeds = seeds_uni)
    mc.crawl(num_threads = 30, time_per_url = 5, request_timeout = 3, proxy_list = [{}])

#    f = open("out", "a")
#    for t in mc.targets:
#        f.write(t.url)

#    for t in mc.targets:
#        with codecs.open("test_output", "a", "utf-8") as temp:
#            temp.write(t.url)
#            temp.write("\n")
#            
#    print len(mc.targets)



import time
import json
import requests
import sys
from urlparse import parse_qs
from urlparse import urlparse
from urlparse import urlunparse
from urllib import urlencode
from multiprocessing import Process, Pool, Queue, current_process
from multiprocessing.pool import ThreadPool
from multiprocessing import TimeoutError
import traceback
from sets import Set
import os 
from bs4 import BeautifulSoup, SoupStrainer
from urlparse import urlparse, urlunparse
from requests.exceptions import ConnectionError
import urllib
from massweb.pnk_net.pnk_request import pnk_request_raw
from massweb.pnk_net.find_post import find_post_requests
from massweb.targets.fuzzy_target import FuzzyTarget
from massweb.targets.target import Target
from massweb.payloads.payload import Payload

class MassRequest(object):

    def __init__(self, num_threads = 10, time_per_url = 10, request_timeout = 10, proxy_list = [{}]):

        self.num_threads = num_threads
        self.time_per_url = time_per_url
        self.request_timeout = request_timeout
        self.proxy_list = proxy_list

        self.results = []
        #!
        self.targets_results = []
        self.urls_finished = []
        self.urls_attempted = []

        self.targets_finished = []
        self.targets_attempted = []
        self.identified_post_requests = []

    def add_to_identified_post(self, x):

        for post_request in x:
            self.identified_post_requests.append(post_request)

    def add_to_finished(self, x):

        self.urls_finished.append(x[0])
        self.results.append(x)

    def add_to_finished_targets(self, x):

        self.targets_finished.append(x[0])
        self.results.append(x)

    def get_urls(self, urls):

        try:
            timeout = float(self.time_per_url * len(urls))
        except:
            url_num = 0
            for url in urls:
                url_num += 1

            timeout = float(self.time_per_url * url_num)
            urls.seek(0)

        pool = Pool(processes = self.num_threads)
        proc_results = []

        for url in urls:
            self.urls_attempted.append(url)
            proc_result = pool.apply_async(func = pnk_request_raw, args = (url, "get", None, self.request_timeout, self.proxy_list), callback = self.add_to_finished)
            proc_results.append(proc_result)

        for pr in proc_results:

            try:
                pr.get(timeout = timeout)

            except:
                pool.terminate()
                pool.join()

        pool.terminate()
        pool.join()
        list_diff = Set(self.urls_attempted).difference(Set(self.urls_finished))
        del self.urls_attempted
        del self.urls_finished

        for url in list_diff:
            #sys.stderr.write("URL %s got timeout" % url)
            self.results.append((url, "__PNK_THREAD_TIMEOUT"))

    def get_targets(self, targets):

        try:
            timeout = float(self.time_per_url * len(targets))
        except:
            url_num = 0
            for url in targets:
                url_num += 1
            targets.seek(0)


        pool = Pool(processes = self.num_threads)
        proc_results = []

        for target in targets:
            if target.ttype == "get":
                self.targets_attempted.append(target)
                proc_result = pool.apply_async(func = pnk_request_raw, args = (target, "get", None, self.request_timeout,  self.proxy_list), callback = self.add_to_finished_targets)
                proc_results.append(proc_result)

        for pr in proc_results:

            try:
                pr.get(timeout = timeout)

            except:
                pool.terminate()
                pool.join()

        pool.terminate()
        pool.join()

        list_diff = Set(self.targets_attempted).difference(Set(self.targets_finished))
        del self.targets_attempted
        del self.targets_finished

        for target in list_diff:
            #sys.stderr.write("URL %s got timeout" % str(target))
            self.targets_results.append((target, "__PNK_THREAD_TIMEOUT"))

    def post_targets(self, targets):

        try:
            timeout = float(self.time_per_url * len(targets))
        except:
            url_num = 0
            for url in targets:
                url_num += 1
            targets.seek(0)

        pool = Pool(processes = self.num_threads)
        proc_results = []

        for target in targets:
            if target.ttype == "post":
                self.targets_attempted.append(target)
                proc_result = pool.apply_async(func = pnk_request_raw, args = (target, "post", target.data, self.request_timeout,  self.proxy_list), callback = self.add_to_finished_targets)
                proc_results.append(proc_result)

        for pr in proc_results:

            try:
                pr.get(timeout = timeout)

            except:
                pool.terminate()
                pool.join()

        pool.terminate()
        pool.join()

        list_diff = Set(self.targets_attempted).difference(Set(self.targets_finished))
        del self.targets_attempted
        del self.targets_finished

        for target in list_diff:
            #sys.stderr.write("URL %s got timeout" % str(target))
            self.targets_results.append((target, "__PNK_THREAD_TIMEOUT"))

    def post_urls(self, urls_and_data):

        try:
            timeout = float(self.time_per_url * len(urls_and_data))
        except:
            url_num = 0
            for url in urls_and_data:
                url_num += 1

            urls_and_data.seek(0)

        pool = Pool(processes = self.num_threads)
        proc_results = []

        for url_and_datum in urls_and_data:
            url = url_and_datum[0]
            datum = url_and_datum[1]

            self.urls_attempted.append(url)
            proc_result = pool.apply_async(func = pnk_request_raw, args = (target, "post", target.data, self.request_timeout,  self.proxy_list), callback = self.add_to_finished_targets)
            proc_results.append(proc_result)

        for pr in proc_results:

            try:
                pr.get(timeout = timeout)

            except:
                pool.terminate()
                pool.join()

        pool.terminate()
        pool.join()
        list_diff = Set(self.urls_attempted).difference(Set(self.urls_finished))

        for url in list_diff:
            self.results.append((url, "__PNK_THREAD_TIMEOUT"))

    def get_post_requests_from_targets(self, targets):

        try:
            timeout = float(self.time_per_url * len(targets))
        except:
            url_num = 0
            for url in targets:
                url_num += 1

            targets.seek(0)

        pool = Pool(processes = self.num_threads)
        proc_results = []

        for target in targets:
            url = target.url
            
            if target.ttype == "get":
                self.targets_attempted.append(target)
                proc_result = pool.apply_async(func = find_post_requests, args = (url, None, True), callback = self.add_to_identified_post)
                proc_results.append(proc_result)
        
        for pr in proc_results:

            try:
                pr.get(timeout = timeout)

            except:
                pool.terminate()
                pool.join()

        pool.terminate()
        pool.join()

    def request_targets(self, targets):

        try:
            timeout = float(self.time_per_url * len(targets))
        except:
            url_num = 0
            for url in targets:
                url_num += 1

            targets.seek(0)

        pool = Pool(processes = self.num_threads)
        proc_results = []

        for target in targets:
            if target.ttype == "post":
                self.targets_attempted.append(target)
                proc_result = pool.apply_async(func = pnk_request_raw, args = (target, "post", target.data, self.request_timeout,  self.proxy_list), callback = self.add_to_finished_targets)
                proc_results.append(proc_result)

            if target.ttype == "get":
                self.targets_attempted.append(target)
                proc_result = pool.apply_async(func = pnk_request_raw, args = (target, "get", None, self.request_timeout,  self.proxy_list), callback = self.add_to_finished_targets)
                proc_results.append(proc_result)

        for pr in proc_results:

            try:
                pr.get(timeout = timeout)

            except:
                pool.terminate()
                pool.join()

        pool.terminate()
        pool.join()

        list_diff = Set(self.targets_attempted).difference(Set(self.targets_finished))
        del self.targets_attempted
        del self.targets_finished

        for target in list_diff:
            #sys.stderr.write("URL %s got timeout" % str(target))
            self.targets_results.append((target, "__PNK_THREAD_TIMEOUT"))

if __name__ == "__main__":

    f = open("urlssmall.txt")
    mr = MassRequest(num_threads = 20, time_per_url = 2)
    
    mr.get_urls(f.readlines())
    print mr.results

#    urls_to_fuzz = ["http://www.pastease.com.au/cart.add?product=33&code=2601BL&price=24.95", "http://www.google.com/", 
#                    "http://www.hyperiongray.com", "http://www.sfgcd.com/ProductsBuy.asp?ProNo=2013-5-3&ProName=%22%3E%3CSCrIpT%3Ealert%287106%29%3C%2FScRiPt%3E"]

#    targ1 = Target(url = "http://www.pastease.com.au/cart.add?product=33&code=2601BL&price=24.95")
#    targ2 = Target(url = "http://www.hyperiongray.com", payload = Payload("fff", ["sqli"]))
#    targ3 = FuzzyTarget(url = "http://www.sfgcd.com/ProductsBuy.asp?ProNo=2013-5-3&ProName=%22%3E%3CSCrIpT%3Ealert%287106%29%3C%2FScRiPt%3E", payload = Payload("fff", ["sqli"]))
#    targ4 = FuzzyTarget(url = "http://www.google.com/", payload = Payload("fff", ["sqli"]))

#    targ1 = Target(url = "http://course.hyperiongray.com/vuln1")

#    targets = [targ1]
#    mr = MassRequest()
#    mr.get_post_requests_from_targets(targets)

#    for t in mr.identified_post_requests:
#        print t

#    targets_to_fuzz = [targ3, targ4]
#    f = open("out_urls_to_fuzz_1mil")
#    targets_to_fuzz = []
#    for line in f:
#        url = line.strip()
#        targets_to_fuzz.append(url)\

#    targ1 = FuzzyTarget(url = "http://course.hyperiongray.com/vuln1eeeeeeeee/formhandler.php", data = {"password" : "%27"}, ttype = "post", payload = Payload("%27", ["sqli"]))
#    targ2 = FuzzyTarget(url = "http://www.hyperiongray.com", payload = Payload("fff", ["sqli"]))

#    targets = [targ1, targ2]
#    mr = MassRequest()
#    mr.request_targets(targets)
#    print mr.results
#
#
#    for r in mr.results:
#        target = r[0]
#        print target, r[1][0:100]
#        print "============================================================================"
#        print target.payload
#        print target.payload.check_type_list
#        print target.payload.payload_str
#        print r[1][0:10]


#    mr.get_urls(urls_to_fuzz)
#    print mr.results
#
#    print "finished"
#    print mr.urls_finished 
#
#    print "results:"
#    for r in mr.results:
#        print r[0], r[1][0:10]
#
#    for x in get_urls(urls_to_fuzz):
#        print x[0], x[1][0:10]



import sys
import time
import json
import requests
import traceback
from massweb.targets.fuzzy_target import FuzzyTarget
from massweb.targets.target import Target
from massweb.proxy_rotator.proxy_rotate import get_random_proxy

def pnk_request_raw(url_or_target, request_type = "get", data = None, req_timeout = 5, proxy_list = [{}]):

    if proxy_list[0]:
        proxy = get_random_proxy(proxy_list)
    else:
        proxy = {}

    try:

        if (isinstance(url_or_target, unicode) or isinstance(url_or_target, str)) and request_type == "get":

            url_or_target = url_or_target.strip()
            r = requests.get(url_or_target, proxies = proxy, timeout = req_timeout)
            return (url_or_target, r)

        if (isinstance(url_or_target, unicode) or isinstance(url_or_target, str)) and request_type == "post":

            url_or_target = url_or_target.strip()
            r = requests.post(url_or_target, data = data, proxies = proxy, timeout = req_timeout)
            return (url_or_target, r)

        if (isinstance(url_or_target, Target)) and request_type == "get":

            r = requests.get(url_or_target.url, proxies = proxy, timeout = req_timeout)
            return (url_or_target, r)

        if (isinstance(url_or_target, Target)) and request_type == "post":

            r = requests.post(url_or_target.url, data = data, proxies = proxy, timeout = req_timeout)
            return (url_or_target, r)

    except:
        #threads suck at exceptions (or I do?), use this to mark failure
        return (url_or_target, "__PNK_REQ_FAILED")



from massweb.payloads.payload import Payload
from massweb.targets.target import Target

class iFuzzer(object):

    def add_payload(self, payload):

        if type(payload) != Payload:
            raise Exception("payload must be of type Payload")

        self.payloads.append(payload)

    def add_payload_from_string(self, payload_str, check_type_list):

        payload = Payload(payload_str, check_type_list)
        self.payloads.append(payload)

    def add_target_from_url(self, url, data = None):

        target = Target(url, data = data)
        self.targets.append(target)

    def add_target(self, target):

        if type(target) != Target:
            raise Exception("target must be of type Target")

        self.targets.append(target)

    def generate_fuzzy_targets(self):
        raise Exception("Not Implemented Error")

    def fuzz(self):
        raise Exception("Not Implemented Error")



class Target(object):

    def __eq__(self, other):
        return self.url == other.url and self.ttype == other.ttype

    def __hash__(self):
        return hash((self.url, self.ttype))

    def __unicode__(self):
        return self.url

    def __str__(self):
        return unicode(self).encode('utf-8', 'replace')

    def __init__(self, url, data = None, ttype = "get"):

        if not isinstance(url, unicode):
            print "exception"
            raise Exception("URL input must be unicode, not string")

        self.url = url
        self.ttype = ttype
        self.data = data

    """
    def replace_param_value(url, param, replacement_string):
        '''Replace a parameter in a url with another string. Returns
        a fully reassembled url as a string.'''

        url_parsed = urlparse(url)
        query_dic = parse_qs(url_parsed.query)
        query_dic[param] = replacement_string

        #this incidentally will also automatically url-encode the payload (thanks urlencode!)
        query_reassembled = urlencode(query_dic, doseq = True)

        #3rd element is always the query, replace query with our own
        url_list_parsed = list(url_parsed)
        url_list_parsed[4] = query_reassembled
        url_parsed_q_replaced = tuple(url_list_parsed)
        url_reassembled = urlunparse(url_parsed_q_replaced)

        return url_reassembled
    """

if __name__ == "__main__":

    t = Target("http://www.hyperiongray.com/")
    print unicode(t)



import os 
from bs4 import BeautifulSoup, SoupStrainer
import sys
from urlparse import urlparse, urlunparse
import traceback
from requests.exceptions import ConnectionError
import urllib
import requests
from massweb.targets.fuzzy_target import FuzzyTarget
from massweb.targets.target import Target
from massweb.pnk_net.pnk_request import pnk_request_raw
from urlparse import urljoin

def normalize_link(url_to_normalize, current_page_url):

    #not quite, doesn't include path in normalization, gets paths wrong

    cp_scheme, cp_netloc, cp_path, cp_params, cp_query, cp_fragment = urlparse(current_page_url)

    parsed_url_to_normalize = urlparse(url_to_normalize)
    scheme, netloc, path, params, query, fragment = urlparse(url_to_normalize)

    if not parsed_url_to_normalize.scheme or not parsed_url_to_normalize.netloc:
        full_url = urljoin(current_page_url, url_to_normalize)
    else:
        full_url = url_to_normalize

    return {"norm_url" : full_url, "netloc" : netloc}

def find_post_requests(url, response_text = None, strict_scope = True):

    if not response_text:
        response_text = pnk_request_raw(url)[1].text

    if strict_scope:
        url_host = urlparse(url).netloc

    post_requests = []
    for form in BeautifulSoup(response_text, 'html.parser', parse_only=SoupStrainer('form')):

        norm_link_dic = normalize_link(form["action"], url)
        norm_url = norm_link_dic["norm_url"]
        form_host = norm_link_dic["netloc"]

        if strict_scope:

            #if form explicitly specifies host that doesn't match current host
            #if doesn't specify host, gets normalized to host so assumed to match
            if form_host and url_host != form_host:
                #print "no host match"
                continue

        listform = ["text", "radio", "checkbox", "password", "file", "image", "hidden"]
        _input = form.findAll('input', {'type' : listform})

        post_data = {}
        for elem in _input:
            input_name = elem["name"]
            try:
                value = urllib.quote_plus(elem["value"])
            except:
                value = ""

            post_data[input_name] = value

        target_post = Target(norm_url, data = post_data, ttype = "post")
        post_requests.append(target_post)

    return post_requests

if __name__ == "__main__":

#    find_post_requests("http://www.gayoutdoors.com/page.cfm?snippetset=yes&amp;typeofsite=snippetdetail&amp;ID=1368&amp;Sectionid=ddd")

#    for p in find_post_requests("http://www.gayoutdoors.com/page.cfm?snippetset=yes&amp;typeofsite=snippetdetail&amp;ID=1368&amp;Sectionid=ddd", strict_scope = True):
#        print p

    for p in find_post_requests("http://course.hyperiongray.com/vuln1"):
        print p, p.data

    for p in find_post_requests("http://course.hyperiongray.com/vuln2/898538a7335fd8e6bac310f079ba3fd1/"):
        print p, p.data



from massweb.vuln_checks.match import match_strings
from massweb.vuln_checks.check import Check

class SQLICheck(Check):

    def __init__(self):

        vuln_strings_raw = ["you have an error in your sql syntax",
        "supplied argument is not a valid mysql",
        "[microsoft][odbc microsoft acess driver]",
        "[microsoft][odbc sql server driver]",
        "microsoft ole db provider for odbc drivers",
        "java.sql.sqlexception: syntax error or access violation",
        "postgresql query failed: error: parser:",
        "db2 sql error:",
        "dynamic sql error",
        "sybase message:",
        "ora-01756: quoted string not properly terminated",
        "ora-00933: sql command not properly ended",
        "pls-00306: wrong number or types",
        "incorrect syntax near",
        "unclosed quotation mark before",
        "syntax error containing the varchar value",
        "ora-01722: invalid number",
        "ora-01858: a non-numeric character was found where a numeric was expected",
        "ora-00920: invalid relational operator",
        "ora-00920: missing right parenthesis"]

        self.vuln_strings = [x.lower() for x in vuln_strings_raw]

    def check(self, content):
        
        content = content.lower()
        return match_strings(content, self.vuln_strings)



import os
from setuptools import setup, find_packages

setup(
    name = 'massweb',
    version = '0.1.15',
    description = 'Fast Web Fuzzing and Scanning',
    long_description = 'Hyperion Gray\'s fast scanning and fuzzing module. Used in PunkSPIDER 3.0.',
    url = 'https://bitbucket.org/acaceres/massweb',
    license = 'Apache 2.0',
    author = 'Alejandro Caceres',
    author_email = 'contact@hyperiongray.com',
    packages = find_packages(),
    include_package_data = True,
    install_requires = ['requests', 'beautifulsoup4', 'html5lib'],
    classifiers = [ "Development Status :: 4 - Beta",
                    'Intended Audience :: Developers',
                    'Programming Language :: Python :: 2.7',
                    'Programming Language :: Python']
)



from massweb.vuln_checks.match import match_strings
from massweb.vuln_checks.check import Check

class MXICheck(Check):

    def __init__(self):

        vuln_strings_raw = ["unexpected extra arguments to select", 
                            "bad or malformed request", "could not access the following folders", 
                            "invalid mailbox name", "go to the folders page"]

        self.vuln_strings = [x.lower() for x in vuln_strings_raw]

    def check(self, content):
        
        content = content.lower()
        return match_strings(content, self.vuln_strings)



import json
from urlparse import urlparse

class Result(object):

    def __unicode__(self):

        url_parsed = urlparse(str(self.fuzzy_target))
        if self.fuzzy_target.ttype == "get":
            to_ret = json.dumps({"url" : unicode(self.fuzzy_target), "results" : self.result_dic, "request_type" : "get", "fuzzy_param" : self.fuzzy_target.fuzzy_param})

        if self.fuzzy_target.ttype == "post":
            to_ret = json.dumps({"url" : unicode(self.fuzzy_target), "data" : self.fuzzy_target.data, "results" : self.result_dic, "request_type" : "post", "fuzzy_param" : self.fuzzy_target.fuzzy_param})

        return to_ret        

    def __str__(self):

        return unicode(self).encode('utf-8', 'replace')

    def __init__(self, fuzzy_target, result_dic = {}):

        self.fuzzy_target = fuzzy_target
        self.result_dic = result_dic



#!/usr/bin/python

from urlparse import urlparse
import sys
from massweb.fuzzers.get_fuzzer import GetFuzzer

def get_proxy_list():

    proxies_raw = """
    ip:username:password
"""

    proxies_list = []
    for x in proxies_raw.split("\n"):
        x = x.strip()
        spl = x.split(":")
        ip = spl[0]
        port = spl[1]
        user = spl[2]
        pw = spl[3]
        val = "http://" + user + ":" + pw + "@" + ip + ":" + port
        _dic = {"http" : val}
        proxies_list.append(_dic)

    return proxies_list

def fuzz_and_print_results(gf):

    for r in gf.fuzz():

        try:
            if True in r.result_dic.values():
                print urlparse(r.fuzzy_target.url).netloc + "\t" + str(r)
        except:
            sys.stderr.write("Failed to fuzz a target!\n")

def create_fuzzer():

    sys.stderr.write("Creating new fuzzer")
    proxy_list = get_proxy_list()
    gfc = GetFuzzer(num_threads = 10, proxy_list = proxy_list)
    gfc.add_payload_from_string("../../../../../../../../../../../../../../../../../../../../etc/passwd#--'@!\\", check_type_list = ["mxi", "sqli", "xpathi", "trav", "osci"])
    gfc.add_payload_from_string('"><ScRipT>alert(31337)</ScrIpT>', check_type_list = ["xss"])
    return gfc

def mapper():

    simul_fuzz = 500
    c = 0
    gf = create_fuzzer()

    for line in sys.stdin:

        try:
            line = line.strip()
            sys.stderr.write("Adding target %s\n" % line)
            gf.add_target_from_url(line)
            c += 1

        except:
            sys.stderr.write("Failed to add line to targets!\n")

        if c == simul_fuzz:
            fuzz_and_print_results(gf)
            del gf.targets
            del gf.payloads
            del gf
            gf = create_fuzzer()
            c = 0

    if c > 0:
        fuzz_and_print_results(gf)

if __name__ == "__main__":

    mapper()




#!/usr/bin/python

import json
import sys
import urlparse
import traceback

previous_domain  = None
previous_result_dic = None
previous_domain_vuln_list = []

for line in sys.stdin:

    try:
        line = line.strip()
        domain_in, result_dic_in = line.split("\t")
        result_dic_in = json.loads(result_dic_in)

        if not previous_domain:
            previous_domain = domain_in
            previous_result_dic = result_dic_in
            previous_domain_vuln_list = []
            continue

        if domain_in == previous_domain:
            previous_domain_vuln_list.append(previous_result_dic)
        else:
            previous_domain_vuln_list.append(previous_result_dic)
            print previous_domain + "\t" +  json.dumps(previous_domain_vuln_list)
            previous_domain_vuln_list = []

        previous_domain = domain_in
        previous_result_dic = result_dic_in

    except:
        traceback.print_exc()
        sys.stderr.write("Line failed to process!")

try:

    if previous_domain == domain_in:
        previous_domain_vuln_list.append(previous_result_dic)
        print previous_domain + "\t" + json.dumps(previous_domain_vuln_list)
except:
    if not previous_domain:
        sys.stderr.write("No results found!")
    else:
        sys.stderr.write("Last line failed to process!")
